# Generated by Django 5.2.9 on 2025-12-16 15:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('directory', '0014_alter_entry_relationship_managers_and_more'),
        ('wbcrm', '0019_alter_activity_companies_alter_activity_participants_and_more'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='product',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='product',
            constraint=models.UniqueConstraint(fields=('slugify_title', 'is_competitor'), name='unique_product'),
        ),
    ]
