# Generated by Django 5.2.9 on 2025-12-09 08:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('directory', '0014_alter_entry_relationship_managers_and_more'),
        ('wbcrm', '0018_activity_search_vector'),
    ]

    operations = [
        migrations.AlterField(
            model_name='activity',
            name='companies',
            field=models.ManyToManyField(blank=True, help_text='The list of companies other than the main company that participate in this activity', related_name='company_participates', through='wbcrm.ActivityCompanyThroughModel', through_fields=('activity', 'company'), to='directory.company', verbose_name='Participating Companies'),
        ),
        migrations.AlterField(
            model_name='activity',
            name='participants',
            field=models.ManyToManyField(blank=True, help_text='The list of participants', related_name='participates', through='wbcrm.ActivityParticipant', through_fields=('activity', 'participant'), to='directory.person', verbose_name='Participating Persons'),
        ),
        migrations.AlterField(
            model_name='product',
            name='prospects',
            field=models.ManyToManyField(blank=True, help_text='The list of prospects', related_name='interested_products', through='wbcrm.ProductCompanyRelationship', through_fields=('product', 'company'), to='directory.company', verbose_name='Prospects'),
        ),
    ]
